@@ -5,7 +5,7 @@ use warnings;
 use 5.010;
 use parent qw/autobox/;
 
-our $VERSION = "1.014";
+our $VERSION = "1.015";
 
 =head1 NAME
 
@@ -23,6 +23,7 @@ autobox::Transform - Autobox methods to transform Arrays and Hashes
 
     $books->grep_by("is_sold_out");
     $books->grep_by(is_in_library => [$library]);
+    $books->grep_by(price => undef, sub { $_ > 56.00 });
 
     $books->uniq_by("id");
 
@@ -72,6 +73,13 @@ autobox::Transform - Autobox methods to transform Arrays and Hashes
     $genre_count->map_each_to_array(sub { "$_: $_[0]" });
     # [ "1: Fantasy", "3: Sci-fi" ]
 
+    # Make the count say "n books"
+    $genre_count->map_each_value(sub { "$_ books" });
+    # {
+    #     "Fantasy" => "1 books",
+    #     "Sci-fi"  => "3 books",
+    # },
+
 
 =head2 Combined examples
 
@@ -301,8 +309,8 @@ package # hide from PAUSE
 sub __invoke_by {
     my $invoke = shift;
     my $array = shift;
-    my( $accessor, $args ) = @_;
-    @_ > 0 or Carp::croak("->${invoke}_by() missing argument: \$accessor");
+    my( $accessor, $args, $subref_name, $subref ) = @_;
+    defined($accessor) or Carp::croak("->${invoke}_by() missing argument: \$accessor");
     @$array or return wantarray ? () : [ ];
 
     if ( ref($array->[0] ) eq "HASH" ) {
@@ -315,12 +323,17 @@ sub __invoke_by {
     ref($args) eq "ARRAY"
         or Carp::croak("${invoke}_by('$accessor', \$args): \$args ($args) is not an array ref");
 
+    if( $subref_name ) {
+        ref($subref) eq "CODE"
+            or Carp::croak("${invoke}_by('$accessor', \$args, \$$subref_name): \$$subref_name ($subref) is not an sub ref");
+    }
+
     my %seen;
     my $invoke_sub = {
         map      => sub { [ CORE::map  { $_->$accessor( @$args ) } @$array ] },
         map_key  => sub { [ CORE::map  { $_->{$accessor}         } @$array ] },
-        grep     => sub { [ CORE::grep { $_->$accessor( @$args ) } @$array ] },
-        grep_key => sub { [ CORE::grep { $_->{$accessor}         } @$array ] },
+        grep     => sub { [ CORE::grep { $subref->( local $_ = $_->$accessor( @$args ) ) } @$array ] },
+        grep_key => sub { [ CORE::grep { $subref->( local $_ = $_->{$accessor}         ) } @$array ] },
         uniq     => sub { [ CORE::grep { ! $seen{ $_->$accessor( @$args ) // "" }++ } @$array ] },
         uniq_key => sub { [ CORE::grep { ! $seen{ $_->{$accessor}         // "" }++ } @$array ] },
     }->{$invoke};
@@ -361,18 +374,17 @@ Or get the hash key value. Examples:
 =cut
 
 sub map_by {
-    return __invoke_by("map", @_);
+    my $array = shift;
+    my ($accessor, $args) = @_;
+    return __invoke_by("map", $array, $accessor, $args);
 }
 
 
 
-=head2 @array->grep_by($accessor, @$args?) : @array | @$array
+=head2 @array->grep_by($accessor, @$args?, $grep_subref = *is_true*) : @array | @$array
 
 Call the $accessor on each object in the list, or get the hash key
-value on each hashref in the list. Like:
-
-    grep { $_->$accessor() }
-    grep { $_->{$accessor} }
+value on each hashref in the list.
 
 Examples:
 
@@ -386,16 +398,45 @@ Examples:
 
     my @books_to_charge_for = $books->grep_by("price_with_tax", [ $tax_pct ]);
 
+Optionally, with the value returned from the $accessor, call
+$grep_subref->($value) to check whether this item should remain in the
+list (default is to check for true values).
+
+The $grep_subref should return a true value to remain. $_ is set to
+the current $value.
+
+Examples:
+
+    my @authors = $authors->grep_by(
+        "publisher", undef,
+        sub { $_->name =~ /Orbit/ },
+    );
+
+    my @authors = $authors->grep_by(
+        publisher_affiliation => [ "with" ],
+        sub { /Orbit / },
+    );
+
+Note: if you do something complicated with the $grep_subref, it might
+be easier and more readable to simply use C<$array->grep()> from
+L<autobox::Core>.
+
 =cut
 
 sub grep_by {
-    return __invoke_by("grep", @_);
+    my $array = shift;
+    my ($accessor, $args, $grep_subref) = @_;
+    $grep_subref //= sub { !! $_ };
+    # grep_by $value, if passed the method value must match the value?
+    return __invoke_by(
+        "grep",
+        $array,
+        $accessor,
+        $args,
+        grep_subref => $grep_subref,
+    );
 }
 
-# grep_by $value, if passed the method value must match the value
-
-
-
 =head2 @array->uniq_by($accessor, @$args?) : @array | @$array
 
 Call the $accessor on each object in the list, or get the hash key
@@ -417,7 +458,9 @@ Examples:
 =cut
 
 sub uniq_by {
-    return __invoke_by("uniq", @_);
+    my $array = shift;
+    my ($accessor, $args) = @_;
+    return __invoke_by("uniq", $array, $accessor, $args);
 }
 
 
@@ -728,6 +771,24 @@ sub map_each {
     return wantarray ? %$new_hash : $new_hash;
 }
 
+=head2 map_each_value($value_subref) : %new_hash | %$new_hash
+
+Map each value in the hash using the $value_subref, but keep the keys
+the same.
+
+C<$value_subref->($key, $value)> is called for each pair (with $_
+set to the value).
+
+The subref should return a single value for each key which will make
+up the %new_hash (with the same keys but with new mapped values).
+
+=head3 Example
+
+    { a => 1, b => 2 }->map_each_value(sub { $_ * 2 });
+    # Returns { a => 2, b => 4 }
+
+=cut
+
 sub map_each_value {
     my $hash = shift;
     my ($value_subref) = @_;
